@@ -29,8 +29,8 @@ void unlock_file_mutex(struct client_s *client);
 gpg_error_t lock_file_mutex(struct client_s *client);
 file_header_internal_t *read_file_header(const gchar *filename, gboolean v1,
 	gpg_error_t *rc);
-gboolean do_compress(assuan_context_t ctx, gint level, gpointer data,
-	guint size, gpointer *out, gulong *outsize, gint *error);
+gpg_error_t do_compress(assuan_context_t ctx, gint level, gpointer data,
+	guint size, gpointer *out, gulong *outsize);
 gboolean valid_filename(const gchar *filename);
 void cleanup_client(struct client_s *client);
 gpg_error_t update_save_flags(const gchar *filename,
