@@ -16,6 +16,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -28,12 +32,10 @@
 #include <string.h>
 #include <ctype.h>
 #include <gpg-error.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "misc.h"
+#include "pwmd_error.h"
+
+extern void log_write(const gchar *fmt, ...);
 
 gboolean strv_printf(gchar ***array, const gchar *fmt, ...)
 {
@@ -47,7 +49,7 @@ gboolean strv_printf(gchar ***array, const gchar *fmt, ...)
 	return FALSE;
 
     if ((a = g_realloc(*array, (len + 2) * sizeof(gchar *))) == NULL) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	return FALSE;
     }
 
@@ -56,7 +58,7 @@ gboolean strv_printf(gchar ***array, const gchar *fmt, ...)
     va_end(ap);
 
     if (ret == -1) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	return FALSE;
     }
 
@@ -134,24 +136,6 @@ gchar *print_fmt(gchar *buf, gsize len, const char *fmt, ...)
     return buf;
 }
 
-gboolean contains_whitespace(const gchar *str)
-{
-    const gchar *p = str;
-    gunichar c;
-    glong len;
-
-    len = g_utf8_strlen(p++, -1) -1;
-
-    while (len--) {
-	c = g_utf8_get_char(p++);
-
-	if (g_unichar_isspace(c))
-	    return TRUE;
-    }
-
-    return FALSE;
-}
-
 gchar *expand_homedir(gchar *str)
 {
     gchar *p = str;
@@ -162,44 +146,6 @@ gchar *expand_homedir(gchar *str)
     return g_strdup(str);
 }
 
-gchar *strip_char(gchar *str, gchar c)
-{
-    gchar *p, *p2;
-
-    for (p = p2 = str; *p; p++) {
-	if (*p == c)
-	    continue;
-
-	*p2++ = *p;
-    }
-
-    *p2 = 0;
-    return str;
-}
-
-gchar *tohex(const guchar *str, gsize len)
-{
-    gchar *p;
-    gint i;
-
-    if (!str || len <= 0)
-	return NULL;
-
-    p = g_malloc0(len*2+1);
-
-    if (!p)
-	return NULL;
-
-    for (i = 0; i < len; i++) {
-	gchar buf[3];
-
-	g_sprintf(buf, "%02x", str[i]);
-	strcat(p, buf);
-    }
-
-    return p;
-}
-
 gchar **split_input_line(gchar *str, gchar *delim, gint n)
 {
     if (!str || !*str)
@@ -226,6 +172,13 @@ gpg_error_t parse_options(gchar **line, struct argv_s *args[], gpointer data)
 	    gchar *tp;
 	    guint len;
 
+	    if (!*p || *p == ' ') {
+		if (*p)
+		    p++;
+
+		break;
+	    }
+
 	    for (tp = opt, len = 0; *p; p++, len++) {
 		if (len+1 == 255)
 		    return GPG_ERR_LINE_TOO_LONG;
@@ -266,9 +219,9 @@ gpg_error_t parse_options(gchar **line, struct argv_s *args[], gpointer data)
 
 	    for (int i = 0; args[i]; i++) {
 		if (!g_strcmp0(args[i]->opt, opt)) {
-		    if (args[i]->type == OPT_NOARG && *value)
+		    if (args[i]->type == OPTION_TYPE_NOARG && *value)
 			return GPG_ERR_SYNTAX;
-		    else if (args[i]->type == OPT_ARG && !*value)
+		    else if (args[i]->type == OPTION_TYPE_ARG && !*value)
 			return GPG_ERR_SYNTAX;
 
 		    rc = args[i]->func(data, value);
