@@ -16,6 +16,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <gcrypt.h>
 #include <glib.h>
@@ -25,10 +29,14 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "mem.h"
+#include "common.h"
+#include "commands.h"
+#include "pinentry.h"
+#include "misc.h"
+#include "pwmd_error.h"
+#include "mutex.h"
+#include "rcfile.h"
 
 #ifdef __FreeBSD__
 #include <sys/types.h>
@@ -40,16 +48,7 @@
 #include "misc.h"
 #endif
 
-#include "mem.h"
-#include "common.h"
-#include "commands.h"
-#include "pinentry.h"
-#include "misc.h"
-#include "pwmd_error.h"
-#include "lock.h"
-#include "rcfile.h"
-
-void free_client_list();
+extern void free_client_list();
 static gpg_error_t set_pinentry_strings(struct pinentry_s *pin, gint which);
 
 static assuan_error_t mem_realloc_cb(void *data, const void *buffer, size_t len)
@@ -717,7 +716,6 @@ int pinentry_iterate(struct client_s *cl, gboolean read_ready)
 	cleanup_client(cl);
 	reset(cl->pinentry);
 	unlock_pin_mutex(cl->pinentry);
-	MUTEX_UNLOCK(&cl->pinentry->status_mutex);
     }
 
     if (cl->pinentry->status == PINENTRY_RUNNING) {
@@ -740,8 +738,6 @@ int pinentry_iterate(struct client_s *cl, gboolean read_ready)
 		    rc = send_error(cl->ctx, pk.error);
 		}
 		else if (pk.status == PINENTRY_PID) {
-		    gint n;
-
 		    /*
 		     * Start the timeout thread for the pinentry process
 		     * now that we know the pid of it.
@@ -752,13 +748,13 @@ int pinentry_iterate(struct client_s *cl, gboolean read_ready)
 		    cl->pinentry->pin_pid = pk.what.pid;
 		    cl->pinentry->tid = pth_spawn(attr, timeout_thread,
 			    cl->pinentry);
-		    n = errno;
+		    rc = gpg_error_from_syserror();
 		    pth_attr_destroy(attr);
 
 		    if (!cl->pinentry->tid) {
 			log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-				_gpg_strerror(gpg_error_from_errno(n)));
-			pk.error = gpg_error_from_errno(n);
+				pwmd_strerror(rc));
+			pk.error = rc;
 		    }
 		}
 		else {
@@ -770,7 +766,7 @@ int pinentry_iterate(struct client_s *cl, gboolean read_ready)
 		    shakey = gcry_malloc(hashlen);
 
 		    if (!shakey)
-			pk.error = gpg_error_from_errno(ENOMEM);
+			pk.error = GPG_ERR_ENOMEM;
 		    else {
 			gcry_md_hash_buffer(GCRY_MD_SHA256, shakey,
 				pk.what.key,
@@ -789,14 +785,14 @@ int pinentry_iterate(struct client_s *cl, gboolean read_ready)
 		    pk.error = GPG_ERR_TIMEOUT;
 
 		pk.error = gpg_err_code_from_syserror();
-		log_write("%s", _gpg_strerror(pk.error));
+		log_write("%s", pwmd_strerror(pk.error));
 	    }
 	    else if (len == 0) {
 		if (cl->pinentry->status == PINENTRY_TIMEOUT)
 		    pk.error = GPG_ERR_TIMEOUT;
 
 		pk.error = GPG_ERR_EOF;
-		log_write("%s", _gpg_strerror(pk.error));
+		log_write("%s", pwmd_strerror(pk.error));
 	    }
 	    else
 		log_write(N_("read(): short byte count"));
