@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <err.h>
-#include <errno.h>
 #include <ctype.h>
 #include <string.h>
 #include <sys/un.h>
@@ -76,7 +75,7 @@
 #include "cache.h"
 #include "misc.h"
 #include "pwmd.h"
-#include "lock.h"
+#include "mutex.h"
 #include "rcfile.h"
 
 GCRY_THREAD_OPTION_PTH_IMPL;
@@ -117,26 +116,17 @@ static void reload_rcfile()
 {
     pth_t tid;
     pth_attr_t attr = pth_attr_new();
-    gint n;
+    gpg_error_t rc;
 
     pth_attr_init(attr);
     pth_attr_set(attr, PTH_ATTR_JOINABLE, 0);
     tid = pth_spawn(attr, reload_rcfile_thread, NULL);
-    n = errno;
+    rc = gpg_error_from_syserror();
     pth_attr_destroy(attr);
 
     if (!tid)
 	log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		_gpg_strerror(gpg_error_from_errno(n)));
-}
-
-gpg_error_t send_syserror(assuan_context_t ctx, gint e)
-{
-    gpg_error_t n = gpg_error_from_errno(e);
-    struct client_s *client = assuan_get_pointer(ctx);
-
-    client->last_rc = e;
-    return assuan_process_done(ctx, assuan_set_error(ctx, n, _gpg_strerror(n)));
+		pwmd_strerror(rc));
 }
 
 gpg_error_t send_error(assuan_context_t ctx, gpg_error_t e)
@@ -196,14 +186,16 @@ void log_write(const gchar *fmt, ...)
     if (!cmdline && logfile) {
 	if ((fd = open(logfile, O_WRONLY|O_CREAT|O_APPEND, 0600)) == -1) {
 	    warn("%s", logfile);
-	    return;
+
+	    if (!log_syslog)
+		return;
 	}
     }
 
     va_start(ap, fmt);
 
     if (g_vasprintf(&args, fmt, ap) == -1) {
-	if (logfile)
+	if (logfile && fd != -1)
 	    close(fd);
 
 	va_end(ap);
@@ -242,13 +234,13 @@ void log_write(const gchar *fmt, ...)
     g_free(args);
 
     if (!line) {
-	if (logfile)
+	if (logfile && fd != -1)
 	    close(fd);
 
 	return;
     }
 
-    if (logfile) {
+    if (logfile && fd != -1) {
 	pth_write(fd, line, strlen(line));
 	fsync(fd);
 	close(fd);
@@ -331,7 +323,7 @@ static gpg_error_t validate_peer(struct client_s *cl)
 
 		if (!buf) {
 		    g_strfreev(users);
-		    return gpg_error_from_errno(ENOMEM);
+		    return GPG_ERR_ENOMEM;
 		}
 
 		if (!getgrnam_r(*(p)+1, &gr, buf, len, &gresult) && gresult) {
@@ -374,7 +366,7 @@ static gpg_error_t validate_peer(struct client_s *cl)
 
 		if (!buf) {
 		    g_strfreev(users);
-		    return gpg_error_from_errno(ENOMEM);
+		    return GPG_ERR_ENOMEM;
 		}
 
 		if (!getpwnam_r(*p, &pw, buf, len, &result) && result) {
@@ -439,7 +431,7 @@ static gboolean new_connection(struct client_s *cl)
     return TRUE;
 
 fail:
-    log_write("%s", _gpg_strerror(rc));
+    log_write("%s", pwmd_strerror(rc));
     return FALSE;
 }
 
@@ -589,7 +581,6 @@ static void *client_thread(void *data)
     struct client_s *cl = g_malloc0(sizeof(struct client_s));
     gpg_error_t rc;
     pth_attr_t attr;
-    gint n;
 
     /*
      * Prevent a race condition with init_new_connection() if this thread
@@ -601,7 +592,7 @@ static void *client_thread(void *data)
     pth_cleanup_push(cleanup_cb, thd);
 
     if (!cl) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	goto fail;
     }
 
@@ -618,14 +609,7 @@ static void *client_thread(void *data)
     cl->pinentry = pinentry_init();
 
     if (!cl->pinentry) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
-	goto fail;
-    }
-#endif
-
-#ifdef HAVE_MLOCKALL
-    if (disable_mlock == FALSE && mlockall(MCL_FUTURE) == -1) {
-	log_write("mlockall(): %s", strerror(errno));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	goto fail;
     }
 #endif
@@ -633,11 +617,11 @@ static void *client_thread(void *data)
     thd->mp = pth_msgport_create(NULL);
     pth_mutex_init(&thd->mp_mutex);
     thd->msg_tid = pth_spawn(NULL, client_msg_thread, thd);
-    n = errno;
+    rc = gpg_error_from_syserror();
 
     if (!thd->msg_tid) {
 	log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		_gpg_strerror(gpg_error_from_errno(n)));
+		pwmd_strerror(rc));
 	goto fail;
     }
 
@@ -645,7 +629,7 @@ static void *client_thread(void *data)
     rc = send_status(cl->ctx, STATUS_CACHE, NULL);
 
     if (rc) {
-	log_write("%s", _gpg_strerror(rc));
+	log_write("%s", pwmd_strerror(rc));
 	goto fail;
     }
 
@@ -689,11 +673,11 @@ static void *client_thread(void *data)
 		if (gpg_err_code(rc) == GPG_ERR_EOF)
 		    goto done;
 
-		log_write("assuan_process_next(): %s", _gpg_strerror(rc));
+		log_write("assuan_process_next(): %s", pwmd_strerror(rc));
 		rc = send_error(cl->ctx, gpg_err_make(PWMD_ERR_SOURCE, rc));
 
 		if (rc) {
-		    log_write("assuan_process_done(): %s", _gpg_strerror(rc));
+		    log_write("assuan_process_done(): %s", pwmd_strerror(rc));
 		    goto done;
 		}
 	    }
@@ -741,7 +725,7 @@ static gchar *do_read_password(const gchar *prompt)
     gchar *key;
 
     if (tcgetattr(STDIN_FILENO, &told) == -1) {
-	log_write("tcgetattr(): %s", strerror(errno));
+	log_write("tcgetattr(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	return NULL;
     }
 
@@ -750,7 +734,7 @@ static gchar *do_read_password(const gchar *prompt)
     tnew.c_lflag |= ICANON|ECHONL;
 
     if (tcsetattr(STDIN_FILENO, TCSANOW, &tnew) == -1) {
-	log_write("tcsetattr(): %s", strerror(errno));
+	log_write("tcsetattr(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	tcsetattr(STDIN_FILENO, TCSANOW, &told);
 	return NULL;
     }
@@ -834,7 +818,7 @@ gpg_error_t export_common(const gchar *filename, struct crypto_s *crypto,
 	gpointer inbuf, gulong insize)
 {
     gpg_error_t rc;
-    gint level, zrc;
+    gint level;
     gulong outsize;
     gpointer outbuf;
 
@@ -848,10 +832,10 @@ gpg_error_t export_common(const gchar *filename, struct crypto_s *crypto,
     if (level < 0)
 	level = 0;
 
-    if (do_compress(NULL, level, inbuf, insize, &outbuf, &outsize, &zrc)
-	    == FALSE) {
-	return zrc == Z_MEM_ERROR ? GPG_ERR_ENOMEM : GPG_ERR_COMPR_ALGO;
-    }
+    rc = do_compress(NULL, level, inbuf, insize, &outbuf, &outsize);
+
+    if (rc)
+	return rc;
 
     crypto->inbuf = outbuf;
     crypto->insize = outsize;
@@ -911,7 +895,7 @@ static gboolean xml_import(const gchar *filename, const gchar *outfile,
     guint hashlen = gcry_md_get_algo_dlen(GCRY_MD_SHA256);
 
     if (stat(filename, &st) == -1) {
-	log_write("%s: %s", filename, strerror(errno));
+	log_write("%s: %s", filename, pwmd_strerror(gpg_error_from_syserror()));
 	return FALSE;
     }
 
@@ -924,7 +908,7 @@ static gboolean xml_import(const gchar *filename, const gchar *outfile,
     memset(crypto->key, 0, hashlen);
 
     if (!crypto->key) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	goto fail;
     }
 
@@ -955,21 +939,20 @@ static gboolean xml_import(const gchar *filename, const gchar *outfile,
     }
 
     if ((fd = open(filename, O_RDONLY)) == -1) {
-	log_write("%s: %s", filename, strerror(errno));
+	log_write("%s: %s", filename, pwmd_strerror(gpg_error_from_syserror()));
 	goto fail;
     }
 
     if ((xmlbuf = gcry_malloc(st.st_size+1)) == NULL) {
 	close(fd);
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	goto fail;
     }
 
     if (pth_read(fd, xmlbuf, st.st_size) == -1) {
-	rc = errno;
+	rc = gpg_error_from_syserror();
 	close(fd);
-	errno = rc;
-	log_write("%s: %s", filename, strerror(errno));
+	log_write("%s: %s", filename, pwmd_strerror(rc));
 	goto fail;
     }
 
@@ -1004,7 +987,7 @@ static gboolean xml_import(const gchar *filename, const gchar *outfile,
     crypto->fh = g_malloc0(sizeof(file_header_internal_t));
 
     if (!crypto->fh) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	goto fail;
     }
 
@@ -1054,7 +1037,7 @@ gboolean do_cache_push(const gchar *filename, const void *password,
     }
 
     if (access(p, R_OK|W_OK) != 0) {
-	log_write("%s: %s", p, strerror(errno));
+	log_write("%s: %s", p, pwmd_strerror(gpg_error_from_syserror()));
 	return FALSE;
     }
 
@@ -1074,7 +1057,7 @@ gboolean do_cache_push(const gchar *filename, const void *password,
     crypto->key = gcry_malloc(hashlen);
 
     if (!crypto->key) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	cleanup_crypto(&crypto);
 	return FALSE;
     }
@@ -1119,7 +1102,7 @@ try_decrypt:
     }
 
     if (cache_update_key(md5file, crypto->key) == FALSE) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	cleanup_crypto(&crypto);
 	return FALSE;
     }
@@ -1135,12 +1118,12 @@ static void init_new_connection(gint fd)
 {
     pth_attr_t attr;
     struct client_thread_s *new;
-    gint n;
+    gpg_error_t rc;
 
     new = g_malloc0(sizeof(struct client_thread_s));
 
     if (!new) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	close(fd);
 	return;
     }
@@ -1151,14 +1134,14 @@ static void init_new_connection(gint fd)
     pth_attr_init(attr);
     pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
     new->tid = pth_spawn(attr, client_thread, new);
-    n = errno;
+    rc = gpg_error_from_syserror();
     pth_attr_destroy(attr);
 
     if (!new->tid) {
 	g_free(new);
 	close(fd);
 	log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		_gpg_strerror(gpg_error_from_errno(n)));
+		pwmd_strerror(rc));
 	MUTEX_UNLOCK(&cn_mutex);
 	return;
     }
@@ -1186,7 +1169,7 @@ static void *accept_thread(void *arg)
 	if (fd == -1) {
 	    if (errno != EAGAIN) {
 		if (!quit) // probably EBADF
-		    log_write("accept(): %s", strerror(errno));
+		    log_write("accept(): %s", pwmd_strerror(gpg_error_from_syserror()));
 
 		break;
 	    }
@@ -1299,18 +1282,18 @@ static void startStopKeepAlive(gboolean term)
 	return;
 
     if (n > 0) {
-	gint e;
 	pth_attr_t attr = pth_attr_new();
+	gpg_error_t rc;
 
 	pth_attr_init(attr);
 	pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
 	keepalive_tid = pth_spawn(attr, keepalive_thread, &n);
-	e = errno;
+	rc = gpg_error_from_syserror();
 	pth_attr_destroy(attr);
 
 	if (!keepalive_tid) {
 	    log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		    _gpg_strerror(gpg_error_from_errno(e)));
+		    pwmd_strerror(rc));
 	    return;
 	}
 
@@ -1363,6 +1346,7 @@ static void server_loop(gint sockfd, gchar **socketpath)
     sigset_t sigset;
     pth_attr_t attr;
     pth_t cache_timeout_tid;
+    gpg_error_t rc;
 
     sigemptyset(&sigset);
 
@@ -1388,13 +1372,17 @@ static void server_loop(gint sockfd, gchar **socketpath)
 
     log_write(N_("%s started for user %s"), PACKAGE_STRING, g_get_user_name());
     log_write(N_("Listening on %s"), *socketpath);
+#ifndef HAVE_SO_PEERCRED
+    log_write(N_("Peer credential checking is NOT supported on this OS."));
+#endif
     attr = pth_attr_new();
     pth_attr_init(attr);
     accept_tid = pth_spawn(attr, accept_thread, &sockfd);
+    rc = gpg_error_from_syserror();
 
     if (!accept_tid) {
 	log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		_gpg_strerror(gpg_error_from_errno(errno)));
+		pwmd_strerror(rc));
 	pth_attr_destroy(attr);
 	goto done;
     }
@@ -1403,10 +1391,11 @@ static void server_loop(gint sockfd, gchar **socketpath)
     startStopKeepAlive(FALSE);
     pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
     cache_timeout_tid = pth_spawn(attr, adjust_cache_timer_thread, NULL);
+    rc = gpg_error_from_syserror();
 
     if (!cache_timeout_tid) {
 	log_write("%s(%i): pth_spawn(): %s", __FILE__, __LINE__,
-		_gpg_strerror(gpg_error_from_errno(errno)));
+		pwmd_strerror(rc));
 	pth_attr_destroy(attr);
 	goto done;
     }
@@ -1424,7 +1413,7 @@ static void server_loop(gint sockfd, gchar **socketpath)
 	    debugfp = fopen(f, "w");
 
 	    if (!debugfp)
-		log_write("%s: %s", f, pwmd_strerror(gpg_error_from_errno(errno)));
+		log_write("%s: %s", f, pwmd_strerror(gpg_error_from_syserror()));
 	    else
 		assuan_set_assuan_log_stream(debugfp);
 
@@ -1664,7 +1653,7 @@ struct crypto_s *init_client_crypto()
     struct crypto_s *new = g_malloc0(sizeof(struct crypto_s));
 
     if (!new) {
-	log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	return NULL;
     }
 
@@ -1885,6 +1874,11 @@ int main(int argc, char *argv[])
 			    "+MEM_DEBUG\n"
 #else
 			    "-MEM_DEBUG\n"
+#endif
+#ifdef MUTEX_DEBUG
+			    "+MUTEX_DEBUG\n"
+#else
+			    "-MUTEX_DEBUG\n"
 #endif
 			    );
 		    exit(EXIT_SUCCESS);
@@ -1961,14 +1955,14 @@ int main(int argc, char *argv[])
 	errno = 0;
 
 	if (setpriority(PRIO_PROCESS, 0, x) == -1) {
-	    log_write("setpriority(): %s", strerror(errno));
+	    log_write("setpriority(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	    goto do_exit;
 	}
     }
 
 #ifdef HAVE_MLOCKALL
-    if (disable_mlock == FALSE && mlockall(MCL_FUTURE) == -1) {
-	log_write("mlockall(): %s", strerror(errno));
+    if (disable_mlock == FALSE && mlockall(MCL_CURRENT|MCL_FUTURE) == -1) {
+	log_write("mlockall(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	goto do_exit;
     }
 #endif
@@ -2028,7 +2022,7 @@ int main(int argc, char *argv[])
     if (argc != optind) {
 	for (; optind < argc; optind++) {
 	    if (strv_printf(&cache_push, "%s", argv[optind]) == FALSE)
-		errx(EXIT_FAILURE, "%s", strerror(ENOMEM));
+		errx(EXIT_FAILURE, "%s", pwmd_strerror(GPG_ERR_ENOMEM));
 	}
     }
 
@@ -2046,7 +2040,7 @@ int main(int argc, char *argv[])
     }
 
     if (chdir(datadir)) {
-	log_write("%s: %s", datadir, strerror(errno));
+	log_write("%s: %s", datadir, pwmd_strerror(gpg_error_from_syserror()));
 	unlink(socketpath);
 	goto do_exit;
     }
@@ -2073,14 +2067,14 @@ int main(int argc, char *argv[])
      * create it then change to datadir.
      */
     if (chdir(socketdir)) {
-	log_write("%s: %s", socketdir, strerror(errno));
+	log_write("%s: %s", socketdir, pwmd_strerror(gpg_error_from_syserror()));
 	goto do_exit;
     }
 
     g_free(socketdir);
 
     if ((sockfd = socket(PF_UNIX, SOCK_STREAM, 0)) == -1) {
-	log_write("socket(): %s", strerror(errno));
+	log_write("socket(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	goto do_exit;
     }
 
@@ -2088,7 +2082,7 @@ int main(int argc, char *argv[])
     g_snprintf(addr.sun_path, sizeof(addr.sun_path), "%s", socketname);
 
     if (bind(sockfd, (struct sockaddr *)&addr, sizeof(struct sockaddr)) == -1) {
-	log_write("bind(): %s", strerror(errno));
+	log_write("bind(): %s", pwmd_strerror(gpg_error_from_syserror()));
 
 	if (errno == EADDRINUSE)
 	    log_write(N_("Either there is another pwmd running or '%s' is a \n"
@@ -2106,7 +2100,7 @@ int main(int argc, char *argv[])
 	g_free(t);
 
 	if (chmod(socketname, mode) == -1) {
-	    log_write("%s: %s", socketname, strerror(errno));
+	    log_write("%s: %s", socketname, pwmd_strerror(gpg_error_from_syserror()));
 	    close(sockfd);
 	    unlink(socketpath);
 	    umask(mask);
@@ -2119,7 +2113,7 @@ int main(int argc, char *argv[])
     g_free(--socketname);
 
     if (chdir(datadir)) {
-	log_write("%s: %s", datadir, strerror(errno));
+	log_write("%s: %s", datadir, pwmd_strerror(gpg_error_from_syserror()));
 	close(sockfd);
 	unlink(socketpath);
 	goto do_exit;
@@ -2128,7 +2122,7 @@ int main(int argc, char *argv[])
     g_free(datadir);
 
     if (listen(sockfd, 0) == -1) {
-	log_write("listen(): %s", strerror(errno));
+	log_write("listen(): %s", pwmd_strerror(gpg_error_from_syserror()));
 	goto do_exit;
     }
 
@@ -2139,7 +2133,7 @@ int main(int argc, char *argv[])
     if (background) {
 	switch (fork()) {
 	    case -1:
-		log_write("fork(): %s", strerror(errno));
+		log_write("fork(): %s", pwmd_strerror(gpg_error_from_syserror()));
 		goto do_exit;
 	    case 0:
 		close(0);
