@@ -27,7 +27,7 @@
 #include <errno.h>
 #include <glib.h>
 #include "pwmd_error.h"
-#include "lock.h"
+#include "mutex.h"
 #include "common.h"
 #include "rcfile.h"
 
@@ -42,9 +42,7 @@ void clear_rcfile_keys()
     groups = g_key_file_get_groups(keyfileh, &n);
 
     for (p = groups; *p; p++) {
-	GError *rc = NULL;
-
-	if (g_key_file_has_key(keyfileh, *p, "key", &rc) == TRUE)
+	if (g_key_file_has_key(keyfileh, *p, "key", NULL) == TRUE)
 	     g_key_file_set_string(keyfileh, *p, "key", "");
     }
 
@@ -58,7 +56,7 @@ static gpg_error_t _getline(struct crypto_s *crypto, const gchar *file)
     gpg_error_t rc = 0;
 
     if (stat(file, &st) == -1)
-	return gpg_error_from_errno(errno);
+	return gpg_error_from_syserror();
 
     crypto->tkey_len = st.st_size;
     crypto->tkey = gcry_malloc(crypto->tkey_len);
@@ -69,7 +67,7 @@ static gpg_error_t _getline(struct crypto_s *crypto, const gchar *file)
     fd = open(file, O_RDONLY);
 
     if (fd == -1)
-	return gpg_error_from_errno(errno);
+	return gpg_error_from_syserror();
 
     gsize len = read(fd, crypto->tkey, crypto->tkey_len);
 
@@ -122,7 +120,7 @@ gpg_error_t parse_rcfile_keyfile(struct crypto_s *crypto, const gchar *filename,
 
 	    if (!t) {
 		g_free(file);
-		log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+		log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 		return GPG_ERR_ENOMEM;
 	    }
 
@@ -145,7 +143,7 @@ gpg_error_t parse_rcfile_keyfile(struct crypto_s *crypto, const gchar *filename,
 	file = g_strdup(filename);
 
 	if (!file) {
-	    log_write("%s(%i): %s", __FILE__, __LINE__, strerror(ENOMEM));
+	    log_write("%s(%i): %s", __FILE__, __LINE__, pwmd_strerror(GPG_ERR_ENOMEM));
 	    return GPG_ERR_ENOMEM;
 	}
     }
@@ -383,24 +381,8 @@ void setup_logging(GKeyFile *kf)
     if (n == TRUE) {
 	gchar *p = g_key_file_get_string(kf, "global", "log_path", NULL);
 
-	if (*p == '~') {
-	    gchar buf[PATH_MAX];
-
-	    p++;
-	    g_snprintf(buf, sizeof(buf), "%s%s", g_get_home_dir(), p--);
-	    g_free(p);
-
-	    if (logfile)
-		g_free(logfile);
-
-	    logfile = g_strdup(buf);
-	}
-	else {
-	    if (logfile)
-		g_free(logfile);
-
-	    logfile = p;
-	}
+	logfile = expand_homedir(p);
+	g_free(p);
     }
 
     log_syslog = g_key_file_get_boolean(kf, "global", "syslog", NULL);
