@@ -16,11 +16,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02110-1301  USA
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <err.h>
-#include <errno.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -28,15 +31,12 @@
 #include <glib.h>
 #include <gcrypt.h>
 #include <libxml/xmlwriter.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "pwmd_error.h"
 #include "misc.h"
 #include "xml.h"
 
+extern void log_write(const gchar *fmt, ...);
+
 /*
  * 'element' must be allocated.
  */
@@ -201,7 +201,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
 	val = xmlNodeGetContent(a->children);
 
 	if (!val) {
-	    rc = gpg_error_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
 
@@ -209,7 +209,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
 
 	if (!tmp) {
 	    xmlFree(val);
-	    rc = gpg_error_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
 
@@ -217,7 +217,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
 
 	if (!tlist) {
 	    xmlFree(val);
-	    rc = gpg_error_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
 
@@ -246,7 +246,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
 	    if (!t) {
 		xmlFree(val);
 		xmlFree(target);
-		rc = gpg_error_from_errno(ENOMEM);
+		rc = GPG_ERR_ENOMEM;
 		goto fail;
 	    }
 
@@ -255,7 +255,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
 	    if (!tlist) {
 		g_free(t);
 		xmlFree(target);
-		rc = gpg_error_from_errno(ENOMEM);
+		rc = GPG_ERR_ENOMEM;
 		goto fail;
 	    }
 	    
@@ -274,7 +274,7 @@ gpg_error_t list_root_elements(xmlDocPtr doc, GString **result,
     string = g_string_new(NULL);
 
     if (!string) {
-	rc = gpg_error_from_errno(ENOMEM);
+	rc = GPG_ERR_ENOMEM;
 	goto fail;
     }
 
@@ -339,7 +339,7 @@ xmlNodePtr create_target_elements_cb(xmlNodePtr node, gchar **path,
 	    n = xmlNewNode(NULL, (xmlChar *)"element");
 
 	    if (!n) {
-		*rc = gpg_error_from_errno(ENOMEM);
+		*rc = GPG_ERR_ENOMEM;
 		return NULL;
 	    }
 
@@ -347,7 +347,7 @@ xmlNodePtr create_target_elements_cb(xmlNodePtr node, gchar **path,
 	    node = xmlAddChild(node, n);
 
 	    if (!node) {
-		*rc = gpg_error_from_errno(ENOMEM);
+		*rc = GPG_ERR_ENOMEM;
 		return NULL;
 	    }
 	}
@@ -436,7 +436,7 @@ xmlNodePtr create_elements_cb(xmlNodePtr node, gchar **elements,
 	    n = xmlNewNode(NULL, (xmlChar *)"element");
 
 	    if (!n) {
-		*rc = gpg_error_from_errno(ENOMEM);
+		*rc = GPG_ERR_ENOMEM;
 		return NULL;
 	    }
 
@@ -448,7 +448,7 @@ xmlNodePtr create_elements_cb(xmlNodePtr node, gchar **elements,
 	    node = xmlAddChild(node, n);
 
 	    if (!node) {
-		*rc = gpg_error_from_errno(ENOMEM);
+		*rc = GPG_ERR_ENOMEM;
 		return NULL;
 	    }
 	}
@@ -470,7 +470,7 @@ xmlNodePtr find_root_element(xmlDocPtr doc, gchar ***req, gpg_error_t *rc,
     gboolean literal = is_literal_element(&root);
 
     if (!root) {
-	*rc = gpg_error_from_errno(ENOMEM);
+	*rc = GPG_ERR_ENOMEM;
 	return NULL;
     }
 
@@ -533,7 +533,7 @@ xmlNodePtr find_root_element(xmlDocPtr doc, gchar ***req, gpg_error_t *rc,
 			 * FIXME ENOMEM
 			 */
 			if (!nreq) {
-			    *rc = gpg_error_from_errno(ENOMEM);
+			    *rc = GPG_ERR_ENOMEM;
 			    return NULL;
 			}
 #endif
@@ -544,7 +544,7 @@ xmlNodePtr find_root_element(xmlDocPtr doc, gchar ***req, gpg_error_t *rc,
 
 			if (!tmp) {
 			    g_free(root);
-			    *rc = gpg_error_from_errno(ENOMEM);
+			    *rc = GPG_ERR_ENOMEM;
 			    return NULL;
 			}
 
@@ -555,7 +555,7 @@ xmlNodePtr find_root_element(xmlDocPtr doc, gchar ***req, gpg_error_t *rc,
 			if (strv_printf(&tmp, "%s", content) == FALSE) {
 			    xmlFree(content);
 			    g_free(root);
-			    *rc = gpg_error_from_errno(ENOMEM);
+			    *rc = GPG_ERR_ENOMEM;
 			    return NULL;
 			}
 
@@ -565,7 +565,7 @@ xmlNodePtr find_root_element(xmlDocPtr doc, gchar ***req, gpg_error_t *rc,
 			g_strfreev(tmp);
 
 			if (!nreq) {
-			    *rc = gpg_error_from_errno(ENOMEM);
+			    *rc = GPG_ERR_ENOMEM;
 			    g_free(root);
 			    return NULL;
 			}
@@ -674,7 +674,7 @@ xmlNodePtr find_elements(xmlDocPtr doc, xmlNodePtr node,
 	gboolean literal;
 
 	if (!t) {
-	    *rc = gpg_error_from_errno(ENOMEM);
+	    *rc = GPG_ERR_ENOMEM;
 	    return NULL;
 	}
 
@@ -701,7 +701,7 @@ xmlNodePtr find_elements(xmlDocPtr doc, xmlNodePtr node,
 	    if (!content) {
 		if (is_list_command == TRUE) {
 		    if (element_to_literal(&(*p)) == FALSE) {
-			*rc = gpg_error_from_errno(ENOMEM);
+			*rc = GPG_ERR_ENOMEM;
 			return NULL;
 		    }
 		}
@@ -774,7 +774,7 @@ xmlNodePtr find_elements(xmlDocPtr doc, xmlNodePtr node,
 		    *zz = g_strdup(*qq++);
 
 		    if (!*zz) {
-			*rc = gpg_error_from_errno(ENOMEM);
+			*rc = GPG_ERR_ENOMEM;
 			n = NULL;
 			break;
 		    }
@@ -832,17 +832,17 @@ static gpg_error_t path_list_recurse(xmlDocPtr doc, xmlNodePtr node,
 	if (elements->verbose) {
 	    if (strv_printf(&elements->elements, "%s\t!%s%s", elements->prefix, a, find_element_node(n->children) ? " 1" : " 0") == FALSE) {
 		xmlFree(a);
-		return gpg_err_code_from_errno(ENOMEM);
+		return GPG_ERR_ENOMEM;
 	    }
 	}
 	else if (strv_printf(&elements->elements, "%s\t!%s", elements->prefix, a) == FALSE) {
 	    xmlFree(a);
-	    return gpg_err_code_from_errno(ENOMEM);
+	    return GPG_ERR_ENOMEM;
 	}
 
 	if (update_element_list(elements) == FALSE) {
 	    xmlFree(a);
-	    return gpg_err_code_from_errno(ENOMEM);
+	    return GPG_ERR_ENOMEM;
 	}
 
 	target = node_has_attribute(n, (xmlChar *)"target");
@@ -887,13 +887,13 @@ static gpg_error_t path_list_recurse(xmlDocPtr doc, xmlNodePtr node,
 		if (strv_printf(&elements->elements, "%s\t%s%s", elements->prefix, a, find_element_node(tnode->children) ? " 1" : " 0") == FALSE) {
 		    xmlFree(a);
 		    xmlFree(target);
-		    return gpg_err_code_from_errno(ENOMEM);
+		    return GPG_ERR_ENOMEM;
 		}
 	    }
 	    else if (strv_printf(&elements->elements, "%s\t%s", elements->prefix, a) == FALSE) {
 		xmlFree(a);
 		xmlFree(target);
-		return gpg_err_code_from_errno(ENOMEM);
+		return GPG_ERR_ENOMEM;
 	    }
 
 	    tmp = g_strjoinv("\t", elements->elements);
@@ -901,14 +901,14 @@ static gpg_error_t path_list_recurse(xmlDocPtr doc, xmlNodePtr node,
 	    if (!tmp) {
 		xmlFree(a);
 		xmlFree(target);
-		return gpg_err_code_from_errno(ENOMEM);
+		return GPG_ERR_ENOMEM;
 	    }
 
 	    if (update_element_list(elements) == FALSE) {
 		g_free(tmp);
 		xmlFree(a);
 		xmlFree(target);
-		return gpg_err_code_from_errno(ENOMEM);
+		return GPG_ERR_ENOMEM;
 	    }
 
 	    if (elements->recurse) {
@@ -938,7 +938,7 @@ children:
 
 	    if (!tmp) {
 		xmlFree(a);
-		return gpg_err_code_from_errno(ENOMEM);
+		return GPG_ERR_ENOMEM;
 	    }
 
 	    elements->prefix = tmp;
@@ -999,7 +999,7 @@ gpg_error_t create_path_list(xmlDocPtr doc, struct element_list_s *elements,
     req_orig = g_strdupv(req);
 
     if (!req_orig) {
-	rc = gpg_err_code_from_errno(ENOMEM);
+	rc = GPG_ERR_ENOMEM;
 	goto fail;
     }
 
@@ -1042,23 +1042,23 @@ gpg_error_t create_path_list(xmlDocPtr doc, struct element_list_s *elements,
 	elements->prefix = g_strjoinv("\t", req_orig);
 
 	if (!elements->prefix) {
-	    rc = gpg_err_code_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
 
 	if (elements->verbose) {
 	    if (strv_printf(&elements->elements, "%s%s", elements->prefix, find_element_node(n->children) ? " 1" : " 0") == FALSE) {
-		rc = gpg_err_code_from_errno(ENOMEM);
+		rc = GPG_ERR_ENOMEM;
 		goto fail;
 	    }
 	}
 	else if (strv_printf(&elements->elements, "%s", elements->prefix) == FALSE) {
-	    rc = gpg_err_code_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
 
 	if (update_element_list(elements) == FALSE) {
-	    rc = gpg_err_code_from_errno(ENOMEM);
+	    rc = GPG_ERR_ENOMEM;
 	    goto fail;
 	}
     }
@@ -1082,7 +1082,7 @@ gpg_error_t recurse_xpath_nodeset(xmlDocPtr doc, xmlNodeSetPtr nodes,
     buf = xmlBufferCreate();
 
     if (!buf)
-	return gpg_err_code_from_errno(ENOMEM);
+	return GPG_ERR_ENOMEM;
 
     for (; value ? i >= 0 : i < nodes->nodeNr; value ? i-- : i++) {
 	xmlNodePtr n = nodes->nodeTab[i];
@@ -1122,6 +1122,7 @@ gpg_error_t recurse_xpath_nodeset(xmlDocPtr doc, xmlNodeSetPtr nodes,
     return 0;
 }
 
+/* Common between a version 1 data file and version < 2.12. */
 static gpg_error_t convert_root_element(xmlNodePtr n)
 {
     xmlChar *a = xmlGetProp(n, (xmlChar *)"_name");
@@ -1156,7 +1157,44 @@ static gpg_error_t convert_root_element(xmlNodePtr n)
     return 0;
 }
 
-/* Updates the DTD and renames the root "accounts" and "account" elements. */
+/* For a version 1 data file. */
+static gpg_error_t convert_elements_recurse_v1(xmlNodePtr n)
+{
+    for (; n; n = n->next) {
+	gpg_error_t rc;
+
+	if (n->type == XML_ELEMENT_NODE &&
+		!xmlStrEqual(n->name, (xmlChar *)"_element")) {
+	    xmlChar *a = xmlGetProp(n, (xmlChar *)"_name");
+
+	    if (a) {
+		xmlFree(a);
+		xmlChar *t = xmlGetNodePath(n);
+
+		log_write(N_("An existing \"_name\" attribute already exists. Please rename this attribute before converting. Path is: %s"), t);
+		xmlFree(t);
+		return GPG_ERR_AMBIGUOUS_NAME;
+	    }
+
+	    rc = add_attribute(n, "_name", (gchar *)n->name);
+
+	    if (rc)
+		return rc;
+
+	    xmlNodeSetName(n, (xmlChar *)"element");
+	}
+
+	rc = convert_elements_recurse_v1(n->children);
+
+	if (rc)
+	    return rc;
+    }
+
+    return 0;
+}
+
+/* Updates the DTD and renames the root "accounts" and "account" elements from
+ * a version 1 data file. */
 gpg_error_t convert_xml(gchar **xml, goffset *len)
 {
     gpg_error_t rc = EPWMD_LIBXML_ERROR;
@@ -1181,6 +1219,13 @@ gpg_error_t convert_xml(gchar **xml, goffset *len)
 		xmlFreeDoc(doc);
 		return rc;
 	    }
+
+	    rc = convert_elements_recurse_v1(n->children);
+
+	    if (rc) {
+		xmlFreeDoc(doc);
+		return rc;
+	    }
 	}
     }
 
@@ -1250,7 +1295,7 @@ static gpg_error_t convert_elements_recurse(xmlDocPtr doc, xmlNodePtr n,
 		xmlChar *tmp = xmlStrdup(n->name);
 
 		if (!tmp)
-		    return gpg_error_from_errno(ENOMEM);
+		    return GPG_ERR_ENOMEM;
 
 		xmlNodeSetName(n, (xmlChar *)"element");
 		rc = add_attribute(n, "_name", (gchar *)tmp);
