@@ -33,13 +33,16 @@ struct element_list_s {
     gchar *prefix;
     gint depth;
     gboolean resolving;
+    gboolean recurse;
+    gboolean verbose;
+    gchar **req;
 };
 
 gint max_recursion_depth;
 
 gpg_error_t new_root_element(xmlDocPtr doc, gchar *name);
 xmlChar *new_document(void);
-gpg_error_t list_root_elements(xmlDocPtr doc, GString **);
+gpg_error_t list_root_elements(xmlDocPtr doc, GString **, gboolean);
 gboolean valid_xml_element(xmlChar *element);
 xmlNodePtr find_elements(xmlDocPtr doc, xmlNodePtr node,
 	gchar **req, gpg_error_t *error, gboolean *target,
@@ -68,5 +71,6 @@ gpg_error_t convert_elements(xmlDocPtr doc);
 gpg_error_t validate_import(xmlNodePtr);
 xmlNodePtr find_element(xmlNodePtr node, gchar *element, xmlNodePtr stop);
 gpg_error_t update_element_mtime(xmlNodePtr n);
+gpg_error_t unlink_node(xmlNodePtr n);
 
 #endif
